/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

var activeCalendarManager = null;
function getCalendarManager() {
	if (!activeCalendarManager) {
		activeCalendarManager = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
	}
	return activeCalendarManager;
}

function calOBMCalendar() {
	this.wrappedJSObject = this;
	try {
		//Init logger
		this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
		var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
		this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
		this.logger.setHeader("calOBMCalendar: ");
		this.logger.debug("new calOBMCalendar()", false, false);

		var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
		if (typeof(UtilsOBM) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
		}

		if (typeof(Calendar_Event_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/synchronizer/Calendar_Event_Synchronizer.js");
		}
		this.mEventSynchronizer = new Calendar_Event_Synchronizer();

		if (typeof(Calendar_Task_Synchronizer) == "undefined") {
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/synchronizer/Calendar_Task_Synchronizer.js");
		}
		this.mTaskSynchronizer = new Calendar_Task_Synchronizer();

	} catch(e) {
		this.logger.error(e, true, false);
	}
}

calOBMCalendar.prototype = {
	//
	// nsISupports interface
	//
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports)
				&& !aIID.equals(Ci.calICalendar)
				&& !aIID.equals(Ci.calICalendarProvider)
				&& !aIID.equals(Ci.nsIRunnable)
				&& !aIID.equals(Ci.calISchedulingSupport)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	//
	// nsIRunnable
	//
	run: function() {
		this.refresh();
	},

	//
	// calICalendar interface
	//
	mID: null,
	get id() {
		return this.mID;
	},
	set id(id) {
		if (this.mID) {
			throw Cr.NS_ERROR_ALREADY_INITIALIZED;
		}
		this.mLocalCalendar.id = id;
		dump("this.mLocalCalendar.id:" + this.mLocalCalendar.id + "\n");
		return (this.mID = id);
	},

	// attribute AUTF8String name;
	get name() {
		var name = getCalendarManager().getCalendarPref_(this, "NAME");
		if (!this.mLocalCalendar.name) {
			//mLocalCalendar.name is needed for light 1.0
			this.mLocalCalendar.name = name;
		}
		return name;
	},
	set name(name) {
		getCalendarManager().setCalendarPref_(this, "NAME", name);
		return (this.mLocalCalendar.name = name);
	},

	// readonly attribute AUTF8String type;
	get type() {
		return "obm";
	},

	// FIXME : needed for light 0.9 
	get superCalendar() {
		// If we have a superCalendar, check this calendar for a superCalendar.
		// This will make sure the topmost calendar is returned
		return (this.mSuperCalendar ? this.mSuperCalendar.superCalendar: this);
	},
	set superCalendar(val) {
		return (this.mSuperCalendar = val);
	},

	// attribute nsIURI uri;
	mUri: null,
	get uri() {
		try {
			return this.mURI;
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},
	set uri(aURI) {
		try {
			if (!this.mURI) {
				dump("set uri" + aURI.spec + "\n");
				this.mURI = aURI;
				this.mOBMName = this.mURI.spec.split(";").pop();
				this.initLocalCalendar(aURI);
			}
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// attribute boolean readOnly;
	mReadOnly: null,
	get readOnly() {
		try {
			//this.logger.debug("get readOnly", false, false);
			if (this.synchronizing) {
				//this.logger.debug(this.mOBMName + "-> synchronizing", false, false);
				return false;
			} else {
				// can't use calendarManager.get/setCalendarPref_() here ! bug #313948
				// have to resort using prefs.js...
				//this.logger.debug(this.mOBMName + "->"+(UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar."
				//+ this.mOBMName + ".readOnly", false)), false, false);
				return UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);
			}
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},
	set readOnly(bool) {
		// nothing, user must not be able to change it
		this.logger.debug("set readOnly (" + bool + ")", false, false);
	},

	// readonly attribute boolean canRefresh;
	get canRefresh() {
		return true;
	},

	// nsIVariant getProperty(in AUTF8String aName);
	getProperty: function(aName) {
		//this.logger.debug("getProperty("+aName+")", false, false);
		if (aName.match("identity")) {
			return this.mLocalCalendar.getProperty(aName);
		} else {
			return this.mLocalCalendar.getProperty(aName);
		}
	},
	// void setProperty(in AUTF8String aName, in nsIVariant aValue);
	setProperty: function(aName, aValue) {
		//this.logger.debug("setProperty("+aName+","+aValue+")", false, false);
		this.mLocalCalendar.setProperty(aName, aValue);
	},
	// void deleteProperty(in AUTF8String aName);
	deleteProperty: function(aName) {
		var calMgr = getCalendarManager();
		calMgr.deleteCalendarPref_(this, aName);
	},

	// void addObserver( in calIObserver observer )
	addObserver: function(aObserver) {
		this.logger.debug("addObserver", false, false);
		try {
			//if ( Ci.calIFreeBusyService )
			if (!this.mObserver) {
				this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
				this.mLocalCalendar.addObserver(this.mObserver);
			}
			// and add lightning observer
			this.mLocalCalendar.addObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void removeObserver( in calIObserver observer );    
	removeObserver: function(aObserver) {
		this.logger.debug("removeObserver", false, false);
		try {
			// an observer is removed, i.e. the calendar is deleted
			// -> we can also remove ours
			if (this.mObserver) {
				this.mLocalCalendar.removeObserver(this.mObserver);
				this.mObserver = null;
			}
			this.mLocalCalendar.removeObserver(aObserver);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void addItem( in calIItemBase aItem, in calIOperationListener aListener );
	addItem: function(aItem, aListener) {
		this.logger.debug("addItem", false, false);
		try {
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			this.mLocalCalendar.addItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void adoptItem( in calIItemBase aItem, in calIOperationListener aListener );
	adoptItem: function(aItem, aListener) {
		try {
			this.logger.debug("adoptItem", false, false);
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			this.mLocalCalendar.adoptItem(aItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void modifyItem( in calIItemBase aNewItem, in calIItemBase aOldItem, 
	//                  in calIOperationListener aListener );
	modifyItem: function(aNewItem, aOldItem, aListener) {
		this.logger.debug("modifyItem (" + this.mOBMName + ")", false, false);
		try {
			if (this.readOnly) {
				throw Ci.calIErrors.CAL_IS_READONLY;
			}
			this.mLocalCalendar.modifyItem(aNewItem, aOldItem, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void deleteItem( in calIItemBase aItem, in calIOperationListener aListener );
	deleteItem: function(aItem, aListener) {
		this.logger.debug("deleteItem (" + this.mOBMName + ")", false, false);
		if (this.readOnly) {
			throw Ci.calIErrors.CAL_IS_READONLY;
		}
		this.mLocalCalendar.deleteItem(aItem, aListener);
	},

	// void getItem( in string aId, in calIOperationListener aListener );
	getItem: function(aId, aListener) {
		try {
			this.logger.debug("getItem (" + this.mOBMName + ")", false, false);
			return this.mLocalCalendar.getItem(aId, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// void getItems( in unsigned long aItemFilter, in unsigned long aCount, 
	//                in calIDateTime aRangeStart, in calIDateTime aRangeEndEx,
	//                in calIOperationListener aListener );
	getItems: function(aItemFilter, aCount, aRangeStart, aRangeEnd, aListener) {
		try {
			this.logger.debug("getItems (" + this.mOBMName + ")", false, false);
			return this.mLocalCalendar.getItems(aItemFilter, aCount, aRangeStart, aRangeEnd, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	// attribute boolean suppressAlarms;
	get suppressAlarms() {
		return false;
	},
	set suppressAlarms(aSuppressAlarms) {
		throw Cr.NS_ERROR_NOT_IMPLEMENTED;
	},

	// readonly attribute boolean sendItipInvitations;
	get sendItipInvitations() {
		return false;
	},

	// void refresh();
	refresh: function() {
		this.logger.debug("try to refresh()", false, false);
		if (UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".firstRefresh", true)) {
			this.logger.debug("refresh canceled", false, false);
			UtilsOBM.setBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".firstRefresh", false);
			return;
		}
		var wWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
		var window = wWatcher.getWindowByName("OBM_SYNCHRO", null);
		if (window != null) {
			this.logger.debug("refresh canceled : sync window open", false, false);
			return;
		}

		var syncManager = Components.classes["@obm.org/syncmanager;1"].createInstance(Components.interfaces.obmISyncManager);
		syncManager.syncCalendar(this);
	},

	// void startBatch();
	startBatch: function() {
		//this.logger.debug(this.mOBMName +" -> start batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
		this.synchronizing = true;
		//this.mLocalCalendar.startBatch();
		this.mReadOnlyStateBeforeBatch = this.mLocalCalendar.readOnly;
		this.mLocalCalendar.removeObserver(this.mObserver);
		this.mLocalCalendar.readOnly = false;
	},

	// void endBatch();
	endBatch: function() {
		if (this.synchronizing === true) {
			this.synchronizing = false;
			this.mLocalCalendar.readOnly = this.mReadOnlyStateBeforeBatch;
			//this.mLocalCalendar.endBatch();
			this.mLocalCalendar.addObserver(this.mObserver);
		}
		//this.logger.debug(this.mOBMName +" -> end batch readOnly:" + this.mLocalCalendar.readOnly, false, false);
	},

	//
	// calICalendarProvider interface
	//
	get prefChromeOverlay() {
		return null;
	},

	get displayName() {
		return "Remote OBM Calendar";
	},

	createCalendar: function() {
		throw NS_ERROR_NOT_IMPLEMENTED;
	},

	deleteCalendar: function(aCalendar, aListener) {
		try {
			this.mLocalCalendar.QueryInterface(Ci.calICalendarProvider).deleteCalendar(this.mLocalCalendar, aListener);
		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

	//
	// calISchedulingSupport
	//
	
	isInvitation: function(aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).isInvitation(aItem);
	},
	
	getInvitedAttendee: function(aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).getInvitedAttendee(aItem);
	},
	
	canNotify: function(aMethod, aItem) {
		return this.mLocalCalendar.superCalendar.QueryInterface(Ci.calISchedulingSupport).canNotify(aMethod, aItem);
	},

	//
	// PRIVATE METHODS
	//
	initLocalCalendar: function(aUri) {
		try {
			this.logger.debug("initLocalCalendar()", false, false);

			this.mLocalCalendar = Cc["@mozilla.org/calendar/calendar;1?type=storage"].createInstance(Ci.calICalendar);
			this.mLocalCalendar.uri = aUri;
			// NOT supported in "storage" calendar, waiting for bug #341776
			// FIXME : fixed in light 0.9
			//this.mLocalCalendar.wrappedJSObject.superCalendar = this;
			this.mObserver = new calOBMObserver(this, this.mOBMName, this.mEventSynchronizer, this.mTaskSynchronizer);
			this.mLocalCalendar.addObserver(this.mObserver);

			this.mLocalCalendar.readOnly = UtilsOBM.getBoolPref("extensions.obm.changes.calICalendar." + this.mOBMName + ".readOnly", false);

		} catch(e) {
			this.logger.error(e, true, false);
		}
	},

};

function calOBMObserver(aOBMCalendar, aOBMName, aEventSynchronizer, aTaskSynchronizer) {
	//Init logger
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("calOBMObserver: ");
	this.logger.debug("calOBMObserver()", false, false);

	this.mOBMCalendar = aOBMCalendar;
	this.mOBMName = aOBMName;
	this.mEventSynchronizer = aEventSynchronizer;
	this.mTaskSynchronizer = aTaskSynchronizer;
}

calOBMObserver.prototype = {

	//
	// calIObserver interface
	//
	onStartBatch: function() {
		this.logger.debug("onStartBatch", false, false);
	},

	onEndBatch: function() {
		this.logger.debug("onEndBatch", false, false);
	},

	onLoad: function() {
		this.logger.debug("onLoad", false, false);
	},

	onAddItem: function(aAddedItem) {
		this.logger.debug("onAddItem", false, false);

		if (aAddedItem instanceof Ci.calIEvent) {
			this.mEventSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
		} else if (aAddedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemAdded(this.mOBMCalendar, aAddedItem);
		}

	},

	onModifyItem: function(aNewItem, aOldItem) {
		this.logger.debug("onModifyItem", false, false);

		if (aNewItem instanceof Ci.calIEvent) {
			this.mEventSynchronizer.notifyItemModified(this.mOBMCalendar, aNewItem);
		} else if (aNewItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemModified(this.mOBMCalendar, aNewItem);
		}
	},

	onDeleteItem: function(aDeletedItem) {
		this.logger.debug("onDeleteItem", false, false);

		if (aDeletedItem instanceof Ci.calIEvent) {
			this.mEventSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
		} else if (aDeletedItem instanceof Ci.calITodo) {
			this.mTaskSynchronizer.notifyItemDeleted(this.mOBMCalendar, aDeletedItem);
		}
	},

	onAlarm: function(aAlarmItem) {
		this.logger.debug("onAlarm", false, false);
	},

	onError: function(aErrNo, aMessage) {
		this.logger.debug("onError", false, false);
	},

	onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {

		this.logger.debug("onPropertyChanged (" + "aName: " + aName + ", aValue: " + aValue + ", aOldValue: " + aOldValue + ")", false, false);
	},

	onPropertyDeleting: function(aCalendar, aName) {
		this.logger.debug("onPropertyChanged (aName: " + aName + ")", false, false);
	}

};

/****
 **** module registration
 ****/

var calOBMCalendarModule = {

	mCID: Components.ID("{86cb9e4f-44c6-4734-9aed-a22795e92b1e}"),
	mContractID: "@mozilla.org/calendar/calendar;1?type=obm",

	registerSelf: function(compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID, "Calendar OBM provider", this.mContractID, fileSpec, location, type);
	},

	getClassObject: function(compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}

		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}

		return this.mFactory;
	},

	mFactory: {
		createInstance: function(outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new calOBMCalendar()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return calOBMCalendarModule;
}